--[[ Backchat
     Frames behind chatwindows

]]--

local addoninfo, bc = ...

local printf = Library.printf.printf

bc.context = nil
bc.frames = {}

function bc.variables_loaded(event, addon)
  if addon == 'Backchat' then
    if not Backchat_settings then
      Backchat_settings = { color = { 0, 0, 0, 1 } }
    end
    bc.settings = Backchat_settings
  end
  if not bc.context then
    bc.context = UI.CreateContext("BackChat")
    bc.context:SetStrata("hud")
    -- behind everything, we hope
    bc.context:SetLayer(0)
  end
  for name, frame in pairs(UI.Native) do
    if name:sub(1, 7) == "Console" then
      local idx = name:sub(8)
      if idx:match("^%d+$") then
	local nidx = tonumber(idx)
        frame:EventAttach(Event.UI.Native.Loaded, function(event) bc.frame_update(nidx, frame, event) end, "backchat_frame_load_" .. idx)
        frame:EventAttach(Event.UI.Layout.Move, function(event) bc.frame_update(nidx, frame, event) end, "backchat_frame_move_" .. idx)
        frame:EventAttach(Event.UI.Layout.Size, function(event) bc.frame_update(nidx, frame, event) end, "backchat_frame_size_" .. idx)
      end
    end
  end
end

function bc.frame_update(idx, frame, event)
  if not bc.frames[idx] then
    bc.frames[idx] = UI.CreateFrame("Frame", "backchat_" .. tostring(idx), bc.context)
  end
  if frame:GetStrata() and frame:GetLoaded() then
    bc.frames[idx]:SetVisible(true)
    bc.frames[idx]:SetPoint("TOPLEFT", UIParent, "TOPLEFT",
      frame:GetLeft() + 1, frame:GetTop() + 30)
    bc.frames[idx]:SetPoint("BOTTOMRIGHT", UIParent, "TOPLEFT",
      frame:GetRight() - 1, frame:GetBottom() - 1)
    bc.frames[idx]:SetBackgroundColor(unpack(bc.settings.color))
  else
    bc.frames[idx]:SetVisible(false)
  end
end

function bc.hexvalues(s, t)
  if not s:match('^%x*$') then
    printf("'#%s' is not a hex color code (looking for #rrggbb[aa]).")
    return nil
  end
  if #s == 2 then
    t[4] = tonumber(s, 16) / 255
  elseif #s == 6 then
    t[1] = tonumber(s:sub(1, 2), 16) / 255
    t[2] = tonumber(s:sub(3, 4), 16) / 255
    t[3] = tonumber(s:sub(5, 6), 16) / 255
    t[4] = 1
  elseif #s == 8 then
    t[1] = tonumber(s:sub(1, 2), 16) / 255
    t[2] = tonumber(s:sub(3, 4), 16) / 255
    t[3] = tonumber(s:sub(5, 6), 16) / 255
    t[4] = tonumber(s:sub(7, 8), 16) / 255
  else
    return nil
  end
  return t
end

function bc.zero_one(s)
  local n = tonumber(s)
  if not n or n < 0 or n > 1 then
    printf("'%s' is not a number between 0 and 1.", s)
    return nil
  end
  return n
end

function bc.slashcommand(args)
  local colors = { unpack(bc.settings.color) }
  local words
  if args and args.leftover_args then
    words = args.leftover_args
  else
    words = {}
  end
  if #words == 1 then
    if words[1]:sub(1, 1) == '#' then
      colors = bc.hexvalues(words[1]:sub(2), colors)
    else
      colors[4] = bc.zero_one(words[1]) or colors[4]
    end
  elseif #words == 3 then
      colors[1] = bc.zero_one(words[1])
      colors[2] = bc.zero_one(words[2])
      colors[3] = bc.zero_one(words[3])
      colors[4] = 1
  elseif #words == 4 then
      colors[1] = bc.zero_one(words[1])
      colors[2] = bc.zero_one(words[2])
      colors[3] = bc.zero_one(words[3])
      colors[4] = bc.zero_one(words[4])
  else
    printf("Usage: /backchat #aa | #rrggbbaa | [r g b] [a]")
    colors = nil
  end
  if colors and colors[1] and colors[2] and colors[3] and colors[4] then
    bc.settings.color = colors
    for idx, frame in pairs(bc.frames) do
      frame:SetBackgroundColor(unpack(colors))
    end
  end
end

Library.LibGetOpt.makeslash("", "Backchat", "backchat", bc.slashcommand)

Command.Event.Attach(Event.Addon.SavedVariables.Load.End, bc.variables_loaded, "variable loaded hook")
